* October 30, 2006
* Author: Rafael Perez Ribas

pr de watt, eclass sortp
vers 9.0

syntax varlist [if] [in] [fw iw aw pw/], [ PScore(varname) VARmatch(string) ///
                                      IDmatch(string) LOGit INdex COMsup BOOTstrap ///
                                      METhod(string) GRaph * ]
token `varlist'

loc Y `1'
loc T `2'

tempvar order
if "`idmatch'" != "" | "`varmatch'" != "" {
   qui g `order' = _n
}

preserve

if `"`if'"' != "" | "`in'" != "" {
   qui keep `if' `in'
}

if "`weight'" != "" & `"`bootstrap'"' != `""' {

di as error "Warning: You are using weights in the estimation of the propensity score together with the bootstrap option."
   exit 198

}

ma s
loc rest `*'

qui drop if `Y'==. | `T'==.


    *If pscore is not user-provided, it has to be estimated

if `"`weight'"' != `""' {
    loc ss "="
}

if `"`pscore'"' == `""'  {

   di in ye _newline(2) " Estimation of the propensity score"

   tempvar PSCORE

    if `"`logit'"' != `""'  {
        logit `rest' [`weight'`ss'`exp'], nolog
    }
    else {
        probit `rest' [`weight'`ss'`exp'], nolog
    }

    if `"`index'"' != `""'  {
        qui predict double `PSCORE', index
    }
    else {
        qui predict double `PSCORE'
    }
}

else {
   loc PSCORE `pscore'
}


    *Region of common support

if `"`comsup'"' != `""'  {
   qui su `PSCORE' if `T'==1
   tempname mintreat maxtreat
   tempvar COMSUP
   sca `mintreat'  = r(min)
   sca `maxtreat'  = r(max)

   di  _newline(2) in ye " Note: the common support option has been selected"
   di  in ye " The region of common support is [" `mintreat' ", " `maxtreat' "]"

   qui {
      g `COMSUP'=(`PSCORE'>=`mintreat' & `PSCORE'<=`maxtreat')
      drop if `COMSUP'!=1
   }
}

if "`method'" == "nnm" | `"`method'"' == `""' {

   di in ye _newline(2) " The program is searching the nearest neighbor of each treated unit. "
   di in ye             " This operation may take a while."


   tempvar ycnew pscoreb pscoref bdiff fdiff pick control fpsdif YCF bpsdif YCB

   if `"`weight'"' != `""' {
       qui egen `ycnew' = wtmean(`Y') , weight(`exp') by(`PSCORE' `T')
   }
   else {
       qui egen `ycnew' = mean(`Y') , by(`PSCORE' `T')
   }

   qui {

          *BACKWARD

      so `PSCORE' `T'

      g `pscoreb'=`PSCORE' if `T'==0
      replace `pscoreb'=`pscoreb'[_n-1] if `pscoreb'==.
      g `bdiff'=abs(`pscoreb'-`PSCORE')


          *FORWARD

      gsort -`PSCORE' `T'

      g `pscoref'=`PSCORE' if `T'==0
      replace `pscoref'=`pscoref'[_n-1] if `pscoref'==.
      g `fdiff'=abs(`pscoref'-`PSCORE')


      g `pick'=uniform()

      replace `fdiff'=100 if `fdiff'==`bdiff' & `T'==1 & `pick'>=0.5
      replace `bdiff'=100 if `fdiff'==`bdiff' & `T'==1 & `pick'<0.5

      g `control'=.


          *Forward Search

      gsort `PSCORE' -`T'

      loc i = 1
      loc fstop = 0
      loc fcount = 0
      g `fpsdif' = .
      g `YCF' = .

      while `fstop' == 0 {
         loc lastfcount = `fcount'
         count if `fpsdif'==. & `T'==1
         loc fcount = r(N)
         if `fcount' == `lastfcount' {
            loc fstop = 1
         }
         else {
            if r(N) != 0 {

               replace `YCF' = `ycnew'[_n + `i'] if `T'==1 & `T'[_n + `i']==0 & `fpsdif'==. & `fdiff'<`bdiff'
               replace `control' = 1 if `T'==0 & `T'[_n - `i']==1 & `fpsdif'[_n-`i']==. & `fdiff'[_n-`i']<`bdiff'[_n-`i']
               replace `fpsdif' = `PSCORE' - `PSCORE'[_n + `i'] if `T'==1 & `T'[_n + `i']==0 & `fpsdif'==.
               loc i = `i' + 1
            }
            else if r(N)== 0 {
               loc fstop = 1
            }
         }
      }


          *Backward Search

      loc i = 1
      loc bstop = 0
      loc bcount = 0
      g `bpsdif' = .
      g `YCB' = .

      while `bstop' == 0 {
         loc lastbcount = `bcount'
         count if `bpsdif'==.&`T'==1
         loc bcount = r(N)
         if `bcount' == `lastbcount' {
            loc bstop = 1
         }
         else {
            if r(N) != 0 {
               replace `YCB' = `ycnew'[_n - `i'] if `T'==1 & `T'[_n - `i']==0 & `bpsdif'==. & `bdiff'<`fdiff'
               replace `control' = 1 if `T'==0 & `T'[_n + `i']==1 & `bpsdif'[_n+`i']==. & `bdiff'[_n+`i']<`fdiff'[_n+`i']
               replace `bpsdif' = `PSCORE' - `PSCORE'[_n - `i'] if `T'==1 & `T'[_n - `i']==0 & `bpsdif'==.
               loc i = `i' + 1
            }
            else if r(N)== 0 {
               loc bstop = 1
            }
         }
      }


          *Choice between backward and forward match

      capture drop YC
      g YC = .

      replace YC = `YCF' if abs(`bpsdif') > abs(`fpsdif') & `T'==1
      replace YC = `YCB' if abs(`bpsdif') < abs(`fpsdif') & `T'==1
      replace YC = `YCB' if abs(`bpsdif') == abs(`fpsdif') & `T'==1 & `pick'<0.5
      replace YC = `YCF' if abs(`bpsdif') == abs(`fpsdif') & `T'==1 & `pick'>=0.5
   }
}

if "`method'" == "kernel" {

   if `bwidth' == 0 {
      local bwidth = 0.06
   }

   di in ye _newline(2) " The program is searching for matches of each treated unit. "
   di in ye " This operation may take a while."

   kernelreg `Y' `PSCORE' if `T'==0
   predict YC if `T'==1
   g `control'=1 if `T'==0

}


    *Result - mean-test

qui {
   count if `T'==1 & YC!=.
   loc nttot = r(N)

   count if `T'==0 & `control'==1
   loc nc = r(N)

   mean `Y' YC if `T'==1 & YC!=. [`weight'`ss'`exp']
   loc myt = _b[`Y']
   loc myc = _b[YC]

   lincom _b[`Y'] - _b[YC]

   eret sca att   = r(estimate)
   eret sca seatt = r(se)
   eret sca tsatt = e(att) / e(seatt)
   eret sca pvatt = 2*ttail(`nttot' - 1, abs(e(tsatt)))
   eret sca yt    = `myt'
   eret sca nt    = `nttot'
   eret sca yc    = `myc'
   eret sca nc    = `nc'

}


    *Display Result

di _newline(3) _column(1)  in ye "ATT estimation with Nearest Neighbor Matching method "
di in ye       _column(1)  "(random draw version)"
di             _column(1)  in ye "Analytical standard errors"
di in gr       _newline(1) in text "{hline 68}"
di             _column(1)  in gr "n. treat." /*
*/             _column(13) in gr "n. contr." /*
*/             _column(25) in gr "      ATT" /*
*/             _column(38) in gr "Std. Err." /*
*/             _column(49) in gr "        t" /*
*/             _column(60) in gr "    P>|t|"
di in gr       in text "{hline 68}"
di _newline(1) _column(1)  in ye %9.0f e(nt) /*
*/             _column(13) in ye %9.0f e(nc) /*
*/             _column(25) in ye %9.3f e(att) /*
*/             _column(38) in ye %9.3f e(seatt) /*
*/             _column(49) in ye %9.3f e(tsatt) /*
*/             _column(60) in ye %9.3f e(pvatt)
di in gr       _newline(1) in text "{hline 68}"
di in gr "Note: the numbers of treated and controls refer to actual"
di in gr "nearest neighbour matches"


    *Export data

if "`idmatch'" != "" & "`varmatch'" != "" {

   qui {

      tempfile matchdta
      g `idmatch' = 0
      replace `idmatch' = 1 if (`T'==0 & `control'==1)
      replace `idmatch' = 1 if (`T'==1 & YC!=.)
      lab var `idmatch' "Observation entered ATT"
      g `varmatch' = YC
      lab var `varmatch' "Matched control outcome"
      so `order'
      sa `matchdta', replace

   }
}

if "`idmatch'" != "" & "`varmatch'" == "" {

   qui {

      tempfile matchdta
      g `idmatch' = 0
      replace `idmatch' = 1 if (`T'==0 & `control'==1)
      replace `idmatch' = 1 if (`T'==1 & YC!=.)
      lab var `idmatch' "Observation entered ATT"
      so `order'
      sa `matchdta', replace

   }
}

if "`idmatch'" == "" & "`varmatch'" != "" {

   qui {

      tempfile matchdta
      g `varmatch' = YC
      lab var `varmatch' "Matched control outcome"
      so `order'
      sa `matchdta', replace

   }
}


restore


    *Bootstrap standard errors

if `"`bootstrap'"' != `""'  {

   di _newline(5) in gr "Bootstrapping of standard errors "

   bs "wattnd `varlist' `if' `in', pscore(`pscore') `logit' `index' `comsup' " att=r(att), nowarn `options'

   eret sca att    = _b[att]
   eret sca bseatt = _se[att]
   eret sca btsatt = e(att) / e(bseatt)
   eret sca bpvatt = 2*ttail(e(nt) - 1, abs(e(btsatt)))

   di _newline(3) _column(1)  in ye "ATT estimation with Nearest Neighbor Matching method"
   di             _column(1)  "(random draw version)"
   di             _column(1)  in ye "Bootstrapped standard errors"
   di in gr       _newline(1) in text "{hline 68}"
   di             _column(1)  in gr "n. treat." /*
   */             _column(13) in gr "n. contr." /*
   */             _column(25) in gr "      ATT" /*
   */             _column(38) in gr "Std. Err." /*
   */             _column(49) in gr "        t" /*
   */             _column(60) in gr "    P>|t|"
   di in gr       in text "{hline 68}"
   di _newline(1) _column(1)  in ye %9.0f e(nt) /*
   */             _column(13) in ye %9.0f e(nc) /*
   */             _column(25) in ye %9.3f e(att) /*
   */             _column(38) in ye %9.3f e(bseatt) /*
   */             _column(49) in ye %9.3f e(btsatt) /*
   */             _column(60) in ye %9.3f e(bpvatt)
   di in gr       _newline(1) in text "{hline 68}"
   di in gr "Note: the numbers of treated and controls refer to actual"
   di in gr "nearest neighbour matches"

}


    *Matching variables saved

if "`idmatch'" != "" | "`varmatch'" != "" {

   qui {

      tempvar _m
      so `order'
      mer `order' using `matchdta', keep(`idmatch' `varmatch') nok _merge(`_m')
      drop `order'

      if "`idmatch'" != "" {
         recode `idmatch' .=0
      }

   }
}

end
