***************************************************
* TECNICAS ECONOMETRICAS PARA AVALIACAO DE IMPACTO
* AULA PRATICA 10
* TRATAMENTO CONTINUO: GENERALIZED PROPENSITY SCORE
* 04/06/2008
***************************************************

clear
set mem 200m
cd "E:\curso"
sysdir set PLUS "E:\curso\ado\plus"
sysdir set PERSONAL "E:\curso\ado\personal"
sysdir set OLDPLACE "E:\curso\ado"

log using log\aula10.log, replace

use http://www.undp-povertycentre.org/evaluation/praticas/bases/nmesdata.dta, clear

* Selecionar somente as variaveis de interesse
keep packyears lastage male race3 beltuse agesmoke educate marital ///
     sregion povstalb totalexp cigsaday yearsince hsqaccwt


* Tranformar variaveis string em numericas
destring agesmoke cigsaday, replace


* Renomear variavel
rename race3 race


* Gerar variaveis de interesse
g lnpackyears = ln(packyears)
g ltotalexp = ln(totalexp)
g agesmoke2 = agesmoke^2
g lastage2 = lastage^2
g posexp = totalexp>0


* Regressao Simples
xi: reg ltotalexp lnpackyears lastage lastage2 male i.race i.beltuse ///
    agesmoke agesmoke2 i.educate i.marital i.sregion i.povstalb [aw=hsqaccwt]


* Primeio passo
xi: reg lnpackyears lastage lastage2 male i.race i.beltuse ///
    agesmoke agesmoke2 i.educate i.marital i.sregion i.povstalb [aw=hsqaccwt]


* Segundo Passo (A): calculando valores preditos
predict index if e(sample)


* Segundo Passo (B): Balanceamento
foreach name in race beltuse educate marital sregion povstalb {
    qui tab `name', g(`name')
}


mat result = ., .
foreach name in lastage lastage2 male agesmoke agesmoke2 race2 race3 ///
     beltuse2 beltuse3 educate2 educate3 educate4 marital2 marital3 ///
     marital4  marital5 sregion2 sregion3 sregion4 povstalb2 povstalb3 ///
     povstalb4 povstalb5 {
        reg `name' lnpackyears [aw=hsqaccwt]
        loc t`name' = _b[lnpackyears]/_se[lnpackyears]
        reg `name' lnpackyears index [aw=hsqaccwt]
        loc t`name'i = _b[lnpackyears]/_se[lnpackyears]
        mat result = result \ `t`name'', `t`name'i'
}

mat list result


* Terceiro Passo
findit quantiles

quantiles index [aw= hsqaccwt] , gencatvar(class3) nquant(3)
quantiles index [aw= hsqaccwt] , gencatvar(class10) nquant(10)


* Quarto Passo (A): 3 subclasses

tab class3 if lnpackyears ~=. & ltotalexp~=.
loc N3 = r(N)
di `N3'

loc beta = 0
forval n = 1/3 {
    reg ltotalexp lnpackyears index if class3==`n'
    loc b`n' = _b[lnpackyears]
    loc se`n' = _se[lnpackyears]
    loc nobs3`n' = e(N)
    di `nobs3`n''
    di `nobs3`n''/`N3'
    loc beta = `beta' + (`b`n'')*`nobs3`n''/`N3'
    di `beta'
}


* Quarto Passo (A): 10 subclasses

tab class10 if lnpackyears ~=. & ltotalexp~=.
loc N10 = r(N)
di `N10'

loc beta = 0
forval n = 1/10 {
    reg ltotalexp lnpackyears index if class10==`n'
    loc b`n' = _b[lnpackyears]
    loc se`n' = _se[lnpackyears]
    loc nobs10`n' = e(N)
    di `nobs10`n''
    di `nobs10`n''/`N10'
    loc beta = `beta' + (`b`n'')*`nobs10`n''/`N10'
    di `beta'
}

log close
