***************************************************
* TECNICAS ECONOMETRICAS PARA AVALIACAO DE IMPACTO
* AULA PRATICA 12
* REVISAO
* 25/06/2008
***************************************************

* Configuraes iniciais

clear
set memory 200m
cd "G:\curso"
sysdir set PLUS "G:\ado\plus"
sysdir set PERSONAL "G:\ado\personal"
sysdir set OLDPLACE "G:\ado"


* Arquivo log

log using log\aula12.log


* Base de dados

use http://www.undp-povertycentre.org/evaluation/praticas/bases/pnadne.dta, clear


* Tabulao descritiva

tabulate pbf
tabulate pbf [aw = v4611]


* Distribuio da renda dos tratados

histogram netypc if pbf==1 [fw = v4611]


* Restrio sobre a amostra da PNAD, somente os mais elegveis de acordo com a renda

keep if netypc<500


* Anlise descritiva do outcome

mean netypc [aw = v4611], over(pbf)

display _b[1] - _b[0]
test _b[1] = _b[0]


* Descritiva das outras caractersticas

mean tamdom-uf9 [aw = v4611], over(pbf)
test 	([nfam]_b[1] = [nfam]_b[0]) ///
	([tamdom]_b[1] = [tamdom]_b[0]) ///
	([idademin]_b[1] = [idademin]_b[0]) ///
	([idosos]_b[1] = [idosos]_b[0]) ///
	([criancas]_b[1] = [criancas]_b[0]) ///
	([pia]_b[1] = [pia]_b[0]) ///
	([mulher]_b[1] = [mulher]_b[0]) ///
	([idade]_b[1] = [idade]_b[0]) ///
	([idade]_b[1] = [idade]_b[0]) ///
	([branco]_b[1] = [branco]_b[0]) ///
	([preto]_b[1] = [preto]_b[0]) ///
	([anoest]_b[1] = [anoest]_b[0]) ///
	([conjuge]_b[1] = [conjuge]_b[0]) ///
	([densidade]_b[1] = [densidade]_b[0]) ///
	([parede]_b[1] = [parede]_b[0]) ///
	([agua]_b[1] = [agua]_b[0]) ///
	([esgoto]_b[1] = [esgoto]_b[0]) ///
	([lixo]_b[1] = [lixo]_b[0]) ///
	([luz]_b[1] = [luz]_b[0]) ///
	([rural]_b[1] = [rural]_b[0]) ///
	([metrop]_b[1] = [metrop]_b[0]) ///
	([cidade]_b[1] = [cidade]_b[0]) ///
	([uf1]_b[1] = [uf1]_b[0]) ///
	([uf2]_b[1] = [uf2]_b[0]) ///
	([uf3]_b[1] = [uf3]_b[0]) ///
	([uf4]_b[1] = [uf4]_b[0]) ///
	([uf5]_b[1] = [uf5]_b[0]) ///
	([uf6]_b[1] = [uf6]_b[0]) ///
	([uf7]_b[1] = [uf7]_b[0]) ///
	([uf8]_b[1] = [uf8]_b[0]) ///
	([uf9]_b[1] = [uf9]_b[0])


* Criando variveis com interao para estimao do ATE

quietly foreach var of varlist nfam-uf8 {
    summarize `var' [aw = v4611] if e(sample)
    generate M_`var' = pbf*(`var' - r(mean))
}


* ATE com ajuste de regresso OLS

regress netypc pbf nfam-uf8 M_* [aw = v4611]

estimates store ATEols


* Criando variveis com interao para estimao do ATT

quietly foreach var of varlist nfam-uf8 {
    summarize `var' [aw = v4611] if pbf==1 & e(sample)
    generate S_`var' = pbf*(`var' - r(mean))
}


* ATT com ajuste de regresso OLS

regress netypc pbf nfam-uf8 S_* [aw = v4611]

estimates store ATTols


* Para verificar o impacto sobre os tratados somente em reas rurais

summarize S_rural if e(sample) & rural==1
scalar Rural = r(mean)

lincom _b[pbf] + _b[S_rural]*Rural


* Seleo de variveis para o Escore de Propenso

global sel = ""
quietly foreach var of varlist nfam-uf9 {
	correlate pbf `var' [aw = v4611]
	local c`var' = abs(_result(4))
	if `c`var''> 0.0105 {
		global sel = "$sel `var'"
	} 
}


* Estimao do Escore de Propenso

logit pbf $sel [pw = v4611]
predict pscore


* Suporte comum

summarize pscore if pbf==1
generate comsup = pscore>=r(min) & pscore<=r(max)
summarize pscore if pbf==0
replace comsup = 0 if pscore<r(min) | pscore>r(max)


* Quantos domiclios esto fora do suporte comum na amostra?

tabulate comsup


* Renda mdia destes domiclios

tabulate comsup [aw = v4611], summarize(netypc)


* Peso para calcular o ATE

generate wate = v4611*((pbf/pscore) + ((1 - pbf)/(1 - pscore))) if comsup==1


* Pesos para calcular o ATT

generate watt = wate*pscore


* Teste da propriedade de balano:

mean pscore tamdom-uf9 [aw = wate], over(pbf)

foreach var of varlist pscore tamdom-uf9 {
    display [`var']_b[1] - [`var']_b[0]
    test ([`var']_b[1] = [`var']_b[0])
}


* ATE calculado com o Escore de Propenso

regress netypc pbf [aw = wate]

estimates store ATEw


* ATT calculado com o Escore de Propenso

regress netypc pbf [aw = watt]

estimates store ATTw


* ATE com regresso ponderada

regress netypc pbf nfam-uf8 M_* [aw = wate]

estimates store ATEwols


* ATT com regresso ponderada

regress netypc pbf nfam-uf8 S_* [aw = watt]

estimates store ATTwols


* Comparando os resultados

estimates table ATEols ATEw ATEwols ATTols ATTw ATTwols, star keep(pbf)


* Regresso Descontnua

* Clculo do bandwidth timo

tempvar h t acum r e w acum2 band

generate `h' = abs(idademin - 15.5)
generate `t' = idademin>15
bysort `t' (`h'): g `acum' = sum(v4611)
bysort `t': g `r' = sum(netypc*v4611)/`acum'
generate `e' = (netypc - `r')^2

sort `h'
generate `acum2' = sum(v4611)
generate `w' = sum(`e'*v4611)/`acum2' if `h'!=.
bysort `t' (`h'): replace `w' = . if `h'<3 | `h'>9

sort `w'
generate `band' = `h'[1]
summarize `band'
scalar BW = r(mean)


* Grfico da diferena fuzzy

twoway lpoly pbf idademin if idademin<=15 [aw = v4611] || ///
       lpoly pbf idademin if idademin>15 [aw = v4611]


* Varivel indicando a fronteira

generate front = idademin<=15 if idademin>=15-BW & idademin<=15+BW


* Teste da diferena na probabilidade de tratamento

regress pbf front [aw = v4611]


* Varivel de controle para o efeito idade

generate x1 = (idademin - 15.5)*(1 - front)
generate x2 = (idademin - 15.5)* front


* Finalmente a regresso MQ2E

ivreg netypc (pbf = front) x1 x2 nfam idosos pia-uf8 [aw = v4611], robust

estimates store RDD


* Compare os resultados

estimates table ATEols ATEw ATEwols ATTols ATTw ATTwols RDD, star keep(pbf)


* Fim

log close


***************************
* End of Do file
***************************
