***************************************************
* TECNICAS ECONOMETRICAS PARA AVALIACAO DE IMPACTO
* AULA PRATICA 3
* AVALIACAO DE IMPACTO POR MEIO DE REGRESSES
* 22/04/2008
***************************************************


* Relembrando a utilizao do profile.do

***************************************************
* Parametros do sistema
***************************************************

clear
set memory 100m
cd "E:\curso"
sysdir set PLUS "E:\ado\plus"
sysdir set PERSONAL "E:\ado\personal"
sysdir set OLDPLACE "E:\ado"


***************************************************
* A base de dados
***************************************************

* Abrindo um arquivo log.

log using log\aula3.log, replace


* Abrir a base pela internet

use http://www.undp-povertycentre.org/evaluation/praticas/bases/FERTIL2.dta, clear



***************************************************
* Anlise descritiva
***************************************************

describe
summarize

* Frequencia da variavel de tratamento

tabulate electric


* Calcular media por tratado e controle

by electric, sort: summarize mnthborn yearborn age radio-evermarr


* Tabulao cruzado entre nmero de filhos e eletricidade

tabulate children electric, row nofreq




***************************************************
* Regresso Linear
***************************************************

*Impacto: regresso linear simples 

regress children electric


* Salvando o resultado a regresso

estimates store OLSimple

* Impacto: regresso linear mltipla

regress children electric age agesq educ urban evermarr

estimates store OLSmult

* Comparando resultados
estimates table OLSimple OLSmult, star



***************************************************
* Estimando o ATE
***************************************************

* Mdia de uma varivel de controle

summarize age if e(sample)


* Criando varivel de interao

generate M_age = electric*(age - r(mean))

* Criando todas as variveis de interao usando um loop

foreach var of varlist agesq educ urban evermarr {
    summarize `var' if e(sample)
    generate M_`var' = electric*(`var' - r(mean))
}

* Incorporando as variveis de interao  regresso

regress children electric age agesq educ urban evermarr M_*

estimates store OLSate



***************************************************
* Estimando o ATT
***************************************************

* Criando as variveis de interao para o att usando quietly e loop 

quietly foreach var of varlist age agesq educ urban evermarr {
    summarize `var' if e(sample) & electric==1
    generate S_`var' = electric*(`var' - r(mean))
}


* Incorporando as novas variveis de interao  regresso

regress children electric age agesq educ urban evermarr S_*
estimates store OLSatt

* Comparando os resultados

estimates table OLSimple OLSmult OLSate OLSatt, star

***************************************************
* Heterogeneidade do Impacto
***************************************************

* Gerando e salvando como uma "local" a mdia de educao para o grupo com menos de 7 anos de estudo

summarize M_educ if e(sample) & educ<7

scalar Educ0 = r(mean)

* Gerando e salvando como uma "local" a mdia de educao para o grupo com  7 anos ou mais de estudo

summarize M_educ if e(sample) & educ>=7

scalar Educ7 = r(mean)

* Chamando novamente a estimao do ATE

estimates restore OLSate

* Testanto o impacto sobre o grupo de menor escolaridade. 

lincom _b[electric] + _b[M_educ]*Educ0

* Testanto o impacto sobre o grupo de maior escolaridade. 

lincom _b[electric] + _b[M_educ]*Educ7

* Testanto se h diferena (absoluta) no impacto entre os dois grupos. 

lincom _b[M_educ]*Educ0 - _b[M_educ]*Educ7


* Testanto se h diferena (relativa/percentual) no impacto entre os dois grupos. 

nlcom (_b[M_educ]*Educ0 - _b[M_educ]*Educ7)/_b[electric]



***************************************************
* Impacto: Outros modelos de regresso
***************************************************

* Regresso de Poisson e efeito marginal


poisson children electric age agesq educ urban evermarr M_*

mfx, varlist(electric)


* Probit ordenado e efeitos marginais

oprobit children electric age agesq educ urban evermarr M_*


* probabilidade de no ter filhos

mfx, varlist(electric) predict(outcome(0))

* probabilidade de ter um filho

mfx, varlist(electric) predict(outcome(1))

* probabilidade de ter mais um filho, dado que j tem 1

mfx, varlist(electric) predict(outcome(2))

* probabilidade de ter mais um filho, dado que j tem 2

mfx, varlist(electric) predict(outcome(3))


*******************************************************
* Impacto sobre a vontade de ter maios ou menos filhos
*******************************************************

* Criando varivel que representa a diferena entre o nmero desejado de filhos e o nmero de filhos

generate vontade = idlnchld - children

histogram vontade

* Criando varivel dummy para anlise do impacto da eletricidade sobre o desejo de ter mais ou menos filhos

generate maisfil = vontade>0 if vontade!=.

generate menosfil = vontade<0 if vontade!=.


* Especificao Probit


probit maisfil electric age agesq educ urban evermarr M_*

mfx, varlist(electric)



* Especificao logit


logit menosfil electric age agesq educ urban evermarr M_*

mfx, varlist(electric)



log close

save base\FERTIL2.dta

exit



