clear
set mem 100m
cd "E:\curso"
sysdir set PLUS "E:\curso\ado\plus"
sysdir set PERSONAL "E:\curso\ado\personal"
sysdir set OLDPLACE "J:\dcp\Cursos\2013-1\Tcnicas_Avanadas_Polticas_Pblicas(DCP098)\curso\ado"

log using log\aula5.log, replace

cd "J:\dcp\Cursos\2013-1\Tcnicas_Avanadas_Polticas_Pblicas(DCP098)\curso"

use base\nswre74_treated.dta, clear

append using base\nswre74_control.dta

tabulate treat

*Vamos comear testando se as variveis em t0 entre os grupos de casos e controles so iguais

mean age-u75, over(treat)

test ([age]_b[1] = [age]_b[0]) ([edu]_b[1] = [edu]_b[0]) ([black]_b[1] = [black]_b[0]) ([hisp]_b[1] = [hisp]_b[0]) ///
([married]_b[1] = [married]_b[0]) ([re74]_b[1] = [re74]_b[0]) ([re75]_b[1] = [re75]_b[0]) ([u75]_b[1] = [u75]_b[0])

test [nodegree]_b[1] = [nodegree]_b[0]

*testes bsicos do impacto do tratamento:

*Inicial
ttest re74, by(treat)

*Final
ttest re78, by(treat)

regress re78 treat

*testes de diferenas-em-diferenas

gen difre=re78-re74

ttest difre, by(treat)
regress difre treat

*testes usando bootstrapping

bootstrap t=r(t), reps(10): ttest re74, by(treat)
bootstrap t=r(t), reps(10): ttest re78, by(treat)
bootstrap, reps(10): regress re78 treat

*Sabemos que o NSW partiu de aleatoriedade, mas suponhamos que no o tivesse, e
*que os indivduos se auto-selecionam. Suponhamos ainda, tal qual no artigo de Abadie et al. (2004), que a auto-seleo
*se d nos observveis age, edu, black, hisp, married, re74, re75, u74, e u75. 
*Como vimos na aula, uma possibilidade para estimar o ATE  rodar a regresso com os observveis e suas interaes:

mean age-u75

gen agedmean = (age - 25.37079)*treat
gen edudmean = (edu - 10.19551)*treat
gen blackdmean = (black - .8337079)*treat
gen hispdmean = (hisp - .0876404)*treat
gen marrieddmean = (married - .1685393)*treat
gen re74dmean =  (re74 - 2102.265)*treat
gen re75dmean = (re75 - 1377.138)*treat
gen u74dmean = (u74 - .2674157)*treat
gen u75dmean = (u75 - .505618)*treat  

***FORMA AUTOMTICA DE CRIAO DAS VARIVEIS

foreach var of varlist age-u75 {
    summarize `var'
    generate `var'dmean = (`var' - r(mean))*treat
}

regress re78 treat age edu black hisp married re74 re75 u74 u75 agedmean edudmean blackdmean hispdmean ///
marrieddmean re74dmean re75dmean u74dmean u75dmean 

*Outra possibilidade  o programa nnmatch, desenvolvido por Abadie et al. (2004). Este programa permite
*uma srie de opes. Eis a sua sintaxe:

*nnmatch depvar treatvar varlist [weight] [if exp] [in range] 
*[, tc({ ate | att | atc }) m(#) metric(maha|matname) exact(varlistex)
*biasadj(bias | varlistadj) robusth(#) population level(#) keep(filename)
*replace]

*Agora, vamos brincar um pouco com o nnmatch. Vamos estimar primeiro o default do algoritmo:

nnmatch re78 treat age edu black hisp married re74 re75 u74 u75

*A partir de agora, estimemos apenas a abordagem DD para este mesmo problema:

nnmatch difre treat age edu black hisp married re74 re75 u74 u75

*Agora, vamos estimar um ATT (somente nos tratados), corrigido pelo vis nas variveis explicativas, casando cada caso com 
*4 controles, arbitrando a matriz de Mahalanobis para a distncia entre as variveis,
*permitindo para heteroscedasticidade entre os grupos de casos e controles.

nnmatch difre treat age edu black hisp married re74 re75 u74 u75, tc(att) ///
m(4) metric(maha) bias(bias) robust(4)

*Podemos forar o mesmo algoritmo para casar indivduos com o mximo de preciso na varivel edu. Isto pode ser feito
*pelo seguinte comando: 

nnmatch difre treat age edu black hisp married re74 re75 u74 u75, tc(att) ///
m(4) metric(maha) bias(bias) robust(4) exact(edu)

*Suponha que queiramos estimar os possveis impactos da expanso do programa para a populao hispnica.
*Vamos, ento, estimar um ATC (somente nos controles), corrigido pelo vis nas variveis explicativas, 
*casando cada caso com 4 controles, arbitrando a matriz de Mahalanobis para a distncia entre as variveis,
*permitindo para heteroscedasticidade entre os grupos de casos e controles. Contudo, a estimativa agora 
*somente para o grupo hispnico.

nnmatch difre treat age edu married re74 re75 u74 u75 if hisp==1, tc(atc) ///
m(4) metric(maha) bias(bias) robust(4)

*O uso de pesos amostrais: muitas das pesquisas que se usa em avaliao de programas sociais so pesquisas amostrais 
*cujas unidades de observao supostamente representam um nmero definido de outras unidades de observao. Isto significa
*que as unidades com maior peso amostral representam mais unidades, e vice versa. O interessante  que o algoritmo de 
*nnmatch tambm trabalha com os pesos amostrais, mesmo que eles sejam "quebrados". Geremos um peso amostral aleatrio
*para ver como o algoritmo se comporta:

/*Criando pesos amostrais "quebrados"*/
set seed 339487731
generate float weightbroken = 1 + uniform()

nnmatch difre treat age edu black hisp married re74 re75 u74 u75 [pweight=weightbroken], tc(att) ///
m(4) metric(maha) bias(bias) robust(4) exact(edu)



 


