clear
set mem 450m
cd "G:\curso"
sysdir set PLUS "G:\curso\ado\plus"
sysdir set PERSONAL "G:\curso\ado\personal"
sysdir set OLDPLACE "G:\curso\ado"

log using log\aula5.log, replace

use base\nswpsid.dta, clear

tabulate TREAT

*Vamos comear testando se as variveis em t0 entre os grupos de casos e controles so iguais

mean AGE-U75, over(TREAT)

test ([AGE]_b[1] = [AGE]_b[0]) ([EDUC]_b[1] = [EDUC]_b[0]) ([BLACK]_b[1] = [BLACK]_b[0]) ([HISP]_b[1] = [HISP]_b[0]) ///
([MARR]_b[1] = [MARR]_b[0]) ([RE74]_b[1] = [RE74]_b[0]) ([RE75]_b[1] = [RE75]_b[0]) ([U75]_b[1] = [U75]_b[0])


*testes bsicos do impacto do tratamento:

*Inicial
ttest RE74, by(TREAT)

*Final
ttest RE78, by(TREAT)

regress RE78 TREAT

*testes de diferenas-em-diferenas

gen difre=RE78-RE74

ttest difre, by(TREAT)
regress difre TREAT

*testes usando bootstrapping

bootstrap t=r(t), reps(10): ttest RE74, by(TREAT)
bootstrap t=r(t), reps(10): ttest RE78, by(TREAT)
bootstrap, reps(10): regress RE78 TREAT

*Sabemos que o NSW o grupo caso-controle partiu de aleatoriedade, mas ao juntarmos a base com o PSID esta aleatoriadade 
*se perde. Contudo, suponhamos que os indviduos se auto-selecionem.
*Suponhamos ainda, tal qual no artigo de Abadie et al. (2004), que a auto-seleo
*se d nos observveis AGE, EDUC, BLACK, HISP, MARR, RE74, RE75, U74, e U75. 
*Como vimos na aula, uma possibilidade para estimar o ATE  rodar a regresso com os observveis e suas interaes:

mean AGE-U75

gen agedmean = (AGE -  34.22579)*TREAT
gen edudmean = (EDUC - 11.99439)*TREAT
gen blackdmean = (BLACK -  .2915888)*TREAT
gen hispdmean = (HISP - .0343925)*TREAT
gen marrieddmean = (MARR - .8194393)*TREAT
gen re74dmean =  (RE74 - 18230)*TREAT
gen re75dmean = (RE75 - 17850.89)*TREAT
gen u74dmean = (U74 -   .1293458)*TREAT
gen u75dmean = (U75 -  .1345794)*TREAT  

regress RE78 TREAT AGE EDUC BLACK HISP MARR RE74 RE75 U74 U75 agedmean edudmean blackdmean hispdmean ///
marrieddmean re74dmean re75dmean u74dmean u75dmean 

regress RE78 TREAT AGE EDUC BLACK HISP MARR RE74 RE75 U74 U75 agedmean edudmean blackdmean hispdmean ///
marrieddmean re74dmean re75dmean u74dmean u75dmean, robust 

regress difre TREAT AGE EDUC BLACK HISP MARR RE74 RE75 U74 U75 agedmean edudmean blackdmean hispdmean ///
marrieddmean re74dmean re75dmean u74dmean u75dmean, robust 

*Outra possibilidade  o programa nnmatch, desenvolvido por Abadie et al. (2004). Este programa permite
*uma srie de opes. Eis a sua sintaxe:

*nnmatch depvar treatvar varlist [weight] [if exp] [in range] 
*[, tc({ ate | att | atc }) m(#) metric(maha|matname) exact(varlistex)
*biasadj(bias | varlistadj) robusth(#) population level(#) keep(filename)
*replace]

*Agora, vamos brincar um pouco com o nnmatch. Vamos estimar primeiro o default do algoritmo:

nnmatch RE78 TREAT AGE EDUC BLACK HISP MARR RE74 RE75 U74 U75

nnmatch RE78 TREAT AGE EDUC BLACK HISP MARR RE74 RE75 U74 U75, tc(att)

*A partir de agora, estimemos apenas a abordagem DD para este mesmo problema:

nnmatch difre TREAT AGE EDUC BLACK HISP MARR RE74 RE75 U74 U75

*Agora, vamos estimar um ATT (somente nos tratados), corrigido pelo vis nas variveis explicativas, casando cada caso com 
*4 controles, arbitrando a matriz de Mahalanobis para a distncia entre as variveis,
*permitindo para heteroscedasticidade entre os grupos de casos e controles.

nnmatch difre TREAT AGE EDUC BLACK HISP MARR RE74 RE75 U74 U75, tc(att) ///
m(4) metric(maha) bias(bias) robust(4)

*Podemos forar o mesmo algoritmo para casar indivduos com o mximo de preciso na varivel EDUC. Isto pode ser feito
*pelo seguinte comando: 

nnmatch difre TREAT AGE EDUC BLACK HISP MARR RE74 RE75 U74 U75, tc(att) ///
m(4) metric(maha) bias(bias) robust(4) exact(EDUC)

*Suponha que queiramos estimar os possveis impactos da expanso do programa para a populao hispnica.
*Vamos, ento, estimar um ATC (somente nos controles), corrigido pelo vis nas variveis explicativas, 
*casando cada caso com 4 controles, arbitrando a matriz de Mahalanobis para a distncia entre as variveis,
*permitindo para heteroscedasticidade entre os grupos de casos e controles. Contudo, a estimativa agora 
*somente para o grupo hispnico.

nnmatch difre TREAT AGE EDUC MARR RE74 RE75 U74 U75 if HISP==1, tc(atc) ///
m(4) metric(maha) bias(bias) robust(4)

*O uso de pesos amostrais: muitas das pesquisas que se usa em avaliao de programas sociais so pesquisas amostrais 
*cujas unidades de observao supostamente representam um nmero definido de outras unidades de observao. Isto significa
*que as unidades com maior peso amostral representam mais unidades, e vice versa. O interessante  que o algoritmo de 
*nnmatch tambm trabalha com os pesos amostrais, mesmo que eles sejam "quebrados". Geremos um peso amostral aleatrio
*para ver como o algoritmo se comporta:

/*Criando pesos amostrais "quebrados"*/
set seed 339487731
generate float weightbroken = 1 + uniform()

nnmatch difre TREAT AGE EDUC BLACK HISP MARR RE74 RE75 U74 U75 [pweight=weightbroken], tc(att) ///
m(4) metric(maha) bias(bias) robust(4) exact(EDUC)

log close

 


