clear
set mem 450m
cd "E:\curso"
sysdir set PLUS "e:\curso\ado\plus"
sysdir set PERSONAL "E:\curso\ado\personal"
sysdir set OLDPLACE "E:\curso\ado"

log using log\aula6.log, replace

use base\nswpsid.dta, clear

tabulate TREAT

*J sabemos o resultado, mas vamos comear testando se as variveis em t0 entre os grupos de casos e controles so iguais

mean AGE-U75, over(TREAT)

test ([AGE]_b[1] = [AGE]_b[0]) ([EDUC]_b[1] = [EDUC]_b[0]) ([BLACK]_b[1] = [BLACK]_b[0]) ([HISP]_b[1] = [HISP]_b[0]) ///
([MARR]_b[1] = [MARR]_b[0]) ([RE74]_b[1] = [RE74]_b[0]) ([RE75]_b[1] = [RE75]_b[0]) ([U75]_b[1] = [U75]_b[0])


*testes bsicos do impacto do tratamento:

*Inicial
ttest RE74, by(TREAT)

*Final
ttest RE78, by(TREAT)

regress RE78 TREAT

*testes de diferenas-em-diferenas

gen difre=RE78-RE74

ttest difre, by(TREAT)
regress difre TREAT

*testes usando bootstrapping

bootstrap t=r(t), reps(10): ttest RE74, by(TREAT)
bootstrap t=r(t), reps(10): ttest RE78, by(TREAT)
bootstrap, reps(10): regress RE78 TREAT
bootstrap t=r(t), reps(10): ttest difre, by(TREAT)
bootstrap, reps(10): regress difre TREAT


*Sabemos que o NSW o grupo caso-controle partiu de aleatoriedade, mas ao juntarmos a base com o PSID esta aleatoriadade 
*se perde. Contudo, suponhamos que os indviduos se auto-selecionem.
*Suponhamos ainda, tal qual no artigo de Abadie et al. (2004), que a auto-seleo
*se d nos observveis AGE, EDUC, BLACK, HISP, MARR, RE74, RE75, U74, e U75. 
*Como vimos na aula passada, uma possibilidade para estimar o ATE  rodar a regresso com os observveis e suas interaes:

mean AGE-U75

gen agedmean = (AGE -  34.22579)*TREAT
gen edudmean = (EDUC - 11.99439)*TREAT
gen blackdmean = (BLACK -  .2915888)*TREAT
gen hispdmean = (HISP - .0343925)*TREAT
gen marrieddmean = (MARR - .8194393)*TREAT
gen re74dmean =  (RE74 - 18230)*TREAT
gen re75dmean = (RE75 - 17850.89)*TREAT
gen u74dmean = (U74 -   .1293458)*TREAT
gen u75dmean = (U75 -  .1345794)*TREAT  

***FORMA AUTOMTICA DE CRIAO DAS VARIVEIS

foreach var of varlist AGE-U75 {
    summarize `var'
    generate `var'dmean = (`var' - r(mean))*TREAT
}

regress RE78 TREAT AGE EDUC BLACK HISP MARR RE74 RE75 U74 U75 agedmean edudmean blackdmean hispdmean ///
marrieddmean re74dmean re75dmean u74dmean u75dmean 

regress difre TREAT AGE EDUC BLACK HISP MARR U74 U75 agedmean edudmean blackdmean hispdmean ///
marrieddmean u74dmean u75dmean, robust 

*Vimos na aula passada que outra possibilidade  o programa nnmatch, desenvolvido por Abadie et al. (2004). 
*Estimemos um ATT para DD com hardmatch, nas variveis acima descritas, com a opo para heteroscedasticidade, distncia
*de Mahalanobis e correo de vis:

nnmatch difre TREAT AGE EDUC BLACK HISP MARR RE74 RE75 U74 U75, tc(att) ///
m(4) metric(maha) bias(bias) robust(4)

*Agora vamos estimar o ATT pelos algoritmos de Becker e Ichino. A nossa base de dados j tem as variveis quadrticas
*e algumas interaes que sero necessrias para replicarmos os modelos do artigo.


*A sintaxe do programa pscore :


*pscore treatment varlist [weight] [if exp] [in range], pscore(newvar) 
*blockid(newvar) detail logit comsup level(#) numblo(#) 

*Testando o algoritmo pscore, sem os pesos, para verificar o artigo do stata journal

pscore TREAT AGE AGESQ EDUC EDUCSQ MARR BLACK HISP RE74 RE75 RE74SQ RE75SQ U74BLACK, ///
pscore(myscore) blockid(myblock) /*detail*/ comsup numblo(5) level (0.005) logit

/* Os argumentos "pscore" e "blockid" so compulsrios, uma vez que eles criaro as variveis da probabilidade estimada 
e do identificador de bloco. O identificador de bloco so as faixas de propensity score sobre as quais os testes de 
mdia ocorrero. Lembre-se que, ao aumentar o nmero de blocos (numblo) tm-se maior chance de se atingir as condies 
de base - basement conditions. A opo detail pode ser interessante se quisermos ver os testes de mdia dentro dos blocos
1 a 1, mas ele j indica em quais variveis esta condio  violada. O level indica o rigor dos testes de hiptese 
de que as variveis do probit ou logit so iguais dentro dos blocos.*/

*Estimemos os efeitos marginais e criemos a varivel da probabilidade estimada

mfx

sum myscore

gen myscoredmean = (myscore -  .0691589)*TREAT

sum myscoredmean

*Vamos rodar o modelo de regresso considerando a prob. Esta estratgia  muito semelhante  adotada na aula anterior

regress RE78 TREAT myscore myscoredmean, robust
regress difre TREAT myscore myscoredmean, robust

*Agora, vamos comear as estimativas do ATT das DD usando PSM. 
*A sintaxe do comando que utilizaremos (ATTR)  

*attr outcome treatment varlist [weight] [if exp] [in range] ,
*pscore(scorevar) logit index radius(#) comsup detail bootstrap
*reps(#) noisily dots 


*Comecemos  ento com um radius matching com distncias 
*de prob no superiores a 0.01, suporte comum, bootstrap com 10 reposies:

attr difre TREAT AGE AGESQ EDUC EDUCSQ MARR BLACK HISP RE74 RE75 RE74SQ RE75SQ U74BLACK,  ///
pscore(myscore) comsup boot reps(10) dots logit radius(0.01)

*Vimos que a literatura aponta que o kernel matching estima a varincia de forma 
*mais robusta que o radius matching. A sintaxe do kernel matching  a seguinte:

*attk outcome treatment varlist [weight] [if exp] [in range] ,
*pscore(scorevar) logit index epan bwidth(#) comsup detail bootstrap
*reps(#) noisily dots 

*A nica diferena de sintaxe em relao ao radius matching so as opes epan e bwidth, que se referem 
* escolha do Kernel de Epanechnikov ao invs do gaussiano, que  o default. A opo bwidth s faz sentido
*com a opo epan, que vai determinar a largura da banda deste Kernel. O default  0,06.
*Estimemos pois o kernel matching, usando o kernel epan :

attk difre TREAT AGE AGESQ EDUC EDUCSQ MARR BLACK HISP RE74 RE75 RE74SQ RE75SQ U74BLACK,  ///
pscore(myscore) comsup epan boot reps(10) dots logit

*Como estes programas lidam com os pesos amostrais? Novamente, criemos pesos aleatrios:

set seed 339487731
generate float weightbroken = 1 + uniform()

attk difre TREAT AGE AGESQ EDUC EDUCSQ MARR BLACK HISP RE74 RE75 RE74SQ RE75SQ U74BLACK [pweight=weightbroken],  ///
pscore(myscore) comsup boot reps(10) dots logit

*Preste ateno  mensagem de erro do programa - de fato, no bootstrapping os pesos amostrais podem perder o sentido.
*Como ele no estima as varincias do kernel matching analiticamente na presena de pesos amostrais,
*A opo  fazer o seguinte, contando que o pscore j levou em conta os pesos amostrais:

pscore TREAT AGE AGESQ EDUC EDUCSQ MARR BLACK HISP RE74 RE75 RE74SQ RE75SQ U74BLACK [pweight=weightbroken], ///
pscore(myscore) blockid(myblock) /*detail*/ comsup numblo(5) level (0.005) logit
attk difre TREAT,  ///
pscore(myscore) comsup epan boot reps(10) dots 


*Radius matching

attr difre TREAT AGE AGESQ EDUC EDUCSQ MARR BLACK HISP RE74 RE75 RE74SQ RE75SQ U74BLACK [pweight=weightbroken],  ///
pscore(myscore) comsup logit radius(0.01)

*Veja que as opes boot, reps() e dots foram removidas.



log close

 


