***************************************************
* TECNICAS ECONOMETRICAS PARA AVALIACAO DE IMPACTO
* AULA PRATICA 1
* INTRODUCAO AO STATA
* 09/04/2008
***************************************************

***************************************************
* Parametros do sistama
***************************************************

clear
set mem 50m
set matsize 500
set more off
cd "E:\stata"

***************************************************
* Utilizando algumas ferramentas
***************************************************

* abrir aquivo .log
log using intro.log, append

* abrir a base de dados
use mroz

* buscar por "sample selection"
search sample selection

* buscar ado file na internet
findit selmlog

* instalar um ado file da internet
ssc install quantiles

* ajuda sobre funes
help function

* comando 'display'
di "Introduo ao Stata"
di 20^.7, exp(.258)

* salvar arquivo de dados
save mroz, replace


***************************************************
* Comandos basicos para manipulacao da base e descricao
***************************************************

* descrio das variveis da base de dados
describe

* listar observaes
list inlf hours wage  kidslt6 in 3/10

* sumario das variveis
sum pea hours wage kidslt6
sum wage if hours>0, detail

* outra modo de descricao
codebook lwage

* contando observacoes
count if wage>10 & city==1

* renomear variavel
rename inlf pea

* criar variavel
gen husage2 = husage^2
gen lhours = ln(hours) if hours>0

* substituir valores das variaveis
replace lhours = -1 if lhours==.

* recodificar variavel
recode educ (5/11 = 1 "5 a 11") ///
            (12 = 2 "12") ///
            (13/max = 3 "13 ou +"), g(cateduc)

* tabulao de variaveis
tab1 pea educ

* tabulao de duas variveis
tab educ pea
tab educ pea, col nofreq

* utilizando 'preserve' e 'restore'
preserve

* 'keep' and 'drop'
keep if hours>0

drop lhours

drop if city~=1

restore


***************************************************
* Graficos
***************************************************

help graph

* pizza (mostrar editor de grafico)
graph pie, over(cateduc) plabel(_all percent) by(pea)

* histogram
histogram hours
histogram hours if hours>0

* Scatter plot
scatter hours wage if hours>0

* Curva suavizada
tw lowess hours wage if hours>0
tw lowess hours wage if hours>0 & city==1 || ///
    lowess hours wage if hours>0 & city==0, lp(dash)

* Funcao Densidade
tw kdensity wage
tw kdensity wage if city==1 || kdensity wage if city==0, lp(dash_dot)

log close
